#!/usr/bin/env python

#use print('message') instead of print 'message' in python 2.7 as well:
from __future__ import print_function

#import pytest
from numpy import array, load, sum, conjugate
from common_functions import search_string, cmp_modes
import os


class Test_BdG():
    """
    Check results of different features with BdG
    """

    def test_26_BdG0_eh_symmetry(self):
        """
        Check if tmat, ginp and gmat have the correct particle hole symmetry

        For this we calculate these matrices in bulk Nb without BdG coupling and check if
        all matrices have this symmetry: t^ee(E+i0+)=-t^hh(-E+i0+)
        """
        path0 = 'test_run26_hybrid_4_1/eh_symm/'
        
        print('Check if t^ee(E+i0+) == -t^hh(E+i0+) :')
        assert os.path.exists(path0+'tmat_atom_001_energ_001.npy')
        tmat = load()
        check_ehsymm(tmat)
        print('\n Check Gref:')
        assert os.path.exists(path0+'ginp_energ_001.npy')
        gref = load(path0+'ginp_energ_001.npy')[:,:,-1]
        check_ehsymm(gref)
        print('\n Check Gmat:')
        assert os.path.exists(path0+'gmat_001_energ_001.npy')
        gmat = load(path0+'gmat_001_energ_001.npy')
        check_ehsymm(gmat)

    def test_27_BdG1_eh_sym_and_gap(self):
        """
        Perform small qdos calculation and check for electron hole symmetry and if there is a gap
        """
        import numpy as np
        path0 = 'test_run27_hybrid_1_1/qdos/'
        ef = float(open(path0+'/potential').readlines()[3].split()[1])
        assert os.path.exists(path0+'/qdos.001.1.dat')
        q = np.loadtxt(path0+'/qdos.001.1.dat')
        ne = len(set(q[:,0])); nq = len(q)//ne
        e = q[:,0].reshape(ne,nq)[:,0]; e = (e-ef)*13.6
        k = q[:,1].reshape(ne,nq)[0,:]
        dqe = q[:,4].reshape(ne,nq)
        q = np.loadtxt(path0+'/qdos.001.1.dat_hh')
        dqh = q[:,4].reshape(ne,nq)
        #subplot(2,1,1)
        #pcolormesh(k, e, dqe, shading='nearest') #'auto', 'nearest' or 'gouraud'
        #colorbar()
        #title('BdG=0, ee')
        #ylabel('E-EF (meV)')
        #subplot(2,1,2)
        #pcolormesh(k, e, dqh, shading='nearest')
        #colorbar()
        #title('BdG=0, hh')

        d = abs(dqe-dqh[::-1])
        dm = d.max()
        print('ehsymm', dm, dm/dqe[d==dm])
        assert any(dm/dqe[d==dm]) < 2e-4
        gz = (dqe[1]).max()/(dqe[0]).max()
        print('gapzero', gz)
        assert gz<0.012


"""
    def test_28_BdG0(self):
        path0 = 'test_run25_hybrid_1_4'
        for pname in ['', 'nosratrick']: #, 'nspin2', 'nspin2_nosratrick', 'nspin2_socscale', 'nspin2_socscale_nosratrick']: # uncomment this list for other run modes, adaprt run script then as well
           testpath = pname
           refpath = testpath+'/ref'
           # first check ee block (hh block needs special test)
           cmp_modes([testpath, refpath], path0 , 10**-12, 10**-12)
           # now check ee/hh consistency and eh,he=0
           check_BdG0_block_charges(path0+'/'+testpath, 5*10**-12)
"""

"""
    def test_26_BdG1(self):
        path0 = 'test_run26_hybrid_1_4'
        for pname in ['', 'nosratrick']: #, 'nspin2', 'nspin2_nosratrick', 'nspin2_socscale', 'nspin2_socscale_nosratrick']: # uncomment for other run modes (see above)
           testpath = path0+'/'+pname
           # now check ee/hh, eh/he consistency etc.
           check_BdG1_block_charges(testpath, 5*10**-8) # higher accuracy can be achieved with better k-integration which would take too long, thus only 5*10**-8

"""

# helper functions


def get_BdG_data(testpath):
    with open(testpath+'/output.000.txt') as f:
        tmptxt = f.readlines()
    itmp = 0; res = []
    while itmp>=0:
        itmp = search_string('Sum of valence charges of atoms', tmptxt)
        if itmp>=0:
            tmpval = tmptxt.pop(itmp)
            tmpval = float(tmpval.split()[-1])
            res.append(tmpval)
    assert len(res)==4 # ee, eh, he, and hh blocks present?
    return res

def check_BdG0_block_charges(testpath, eps=10**-14):
    res = get_BdG_data(testpath)
    # check if charge in 4 blocks are consistent
    assert abs(res[0]-res[-1])<eps # ee==hh ?
    assert abs(res[1]-res[2])<eps  # eh==he ?
    assert abs(res[0])>eps         # ee>0 ?
    assert abs(res[1])<eps         # eh==0 ?

def check_BdG1_block_charges(testpath, eps=10**-14):
    res = get_BdG_data(testpath)
    # check if charge in 4 blocks are consistent
    assert abs(res[0]-res[-1])<eps # ee==hh ?
    #assert abs(res[1]-res[2])<eps  # eh==he ?
    assert abs(res[0])>eps         # ee>0 ?
    assert abs(res[1])>eps         # eh>0 ?
    # TODO add some more checks which varify that BdG mode is running correctly
    # maybe different pairing potentials etc.
    # here check atom resolved contributions ...

def check_ehsymm(m):
   d = sum(abs(m[:16,:16]+conjugate(m[16:32,16:32])))
   print('absdiff, reldiff=', d, d/sum(abs(m[:16,:16])), '(threshold=1e-13)')
   assert d<10**-13
